// Unit SoundFX
(**************************************************

- Enthlt: TSoundManager

- Bentigt: bass.dll

**************************************************)

unit F2D_SoundFX;

interface

uses
  // Bentigte Units
  SysUtils, Forms, classes, IniFiles, Bass,
  F2D_BugTracker;

type
  // Klasse zum SoundManagement
  TSoundManager = class
    Sounds: THashedStringList;   // Liste fr die Sounds zum einfacheren Zugriff
    BugTracker: TBugTracker;        // BugTracker (siehe UBugTracker)

    constructor Create;
    destructor Destroy; override;
    procedure Free;
  private
    function GetCount: integer;
  public
    function GetSoundIndex(pSoundname: string): integer;
    function GetSoundName(pIndex: integer): string;

    // Fgt einen Sound hinzu
    procedure AddSound(pFileName, pSoundName: String);

    //Fgt smtliche Sounds aus einem Verzeichnis hinzu (Filter: *.*)
    procedure AddSoundsInDir(pDirName, pFileMask: String; pSubDirs: Boolean);

    //Gibt den Namen eines zuflligen Sounds zurck
    function GetRandomSound: string;

    // Spielt einen Sound ab
    // pLoop - ob der Sound wiederholt werden soll
    // pWait - ob gewartet werden soll, dass der Sound zuende gespielt wurde
    // Jeder Sound kann nur einmal zur Zeit gespielt werden
    function PlaySound(pSoundName: String; pLoop, pWait: Boolean): boolean;

    function PauseSound(pSoundName: String): boolean;  // Pausiert einen Sound
    function StopSound(pSoundName: String): boolean;   // Stoppt einen Sound

    //Spielt einen zuflligen Sound aus einer Liste von Soundnamen ab
    //und liefert den Namen des gewhlten sounds
    function PlayRandomFromList(pSoundNames: TStrings; pLoop, pWait: boolean): string;
    property Count: integer read GetCount;
  end;

implementation

constructor TSoundManager.Create;
begin
  inherited Create;

  // Bass.dll initialisieren
  BASS_Init(-1, 44100, 0, Application.Handle, nil);

  Sounds := THashedStringList.Create;
end;

destructor TSoundManager.Destroy;
begin
  Sounds.Free;

  // Bass.dll freigeben
  BASS_Free();

  inherited Destroy;
end;

procedure TSoundManager.Free;
begin
  if Self <> nil then
    Destroy;
end;

function TSoundManager.GetCount: integer;
begin
  result := Sounds.Count;
end;

function TSoundManager.GetSoundIndex(pSoundname: string): integer;
begin
  result := Sounds.IndexOf(pSoundName);
end;

function TSoundManager.GetSoundName(pIndex: integer): string;
begin
  result := Sounds[pIndex];
end;

procedure TSoundManager.AddSound(pFileName, pSoundName: String);
begin
  // Prfen, ob der Sound nicht schon in der Liste ist
  if (GetSoundIndex(pSoundName) = -1) then
    if (FileExists(pFileName)) then
    begin
      // Sound hinzufgen
      Sounds.AddObject(pSoundName,
      Pointer(BASS_StreamCreateFile(False, PChar(pFileName), 0, 0, BASS_SAMPLE_FX)));

      //BugTracker zum Log Datei schreiben veranlassen (O bedeutet erfolgreich)
      BugTracker.Write('O Sound: ' + pSoundName);
      BugTracker.Write('- Datei: ' + pFileName);
      BugTracker.Write('');
    end
    else
    begin
      // BugTracker zum Log Datei schreiben veranlassen (X bedeutet fehlgeschlagen)
      BugTracker.Write('X Sound: ' + pSoundName);
      BugTracker.Write('- Datei: ' + pFilename);
      BugTracker.Write('');
    end;
end;

// Rekursive Dateisuche - Der Name des Sounds entspricht dann dem Dateinamen
procedure TSoundManager.AddSoundsInDir(pDirName, pFileMask: String; pSubDirs: Boolean);
  procedure ScanDir(const Directory: String);
    function CutExt(aStr: String): String;
    var
      I: Integer;
    begin
      Result := aStr;
      for I := Length(aStr) downto 1 do
        if aStr[i] = '.' then
        begin
          Result := Copy(aStr, 1, I - 1);
          Exit;
        end;
    end;
  var
    SR: TSearchRec;
  begin
    if FindFirst(Directory + pFileMask, faAnyFile - faDirectory, SR) = 0 then
    try
      repeat
        AddSound(Directory + SR.Name, CutExt(SR.Name));
      until FindNext(SR) <> 0;
    finally
      FindClose(SR);
    end;
    if pSubDirs then
    begin
      if FindFirst(Directory + '*.*', faAnyFile, SR) = 0 then
      try
        repeat
          if ((SR.Attr AND faDirectory) = faDirectory) AND
              (SR.Name <> '.') AND (SR.Name <> '..') then
            ScanDir(Directory + SR.Name + '\');
        until FindNext(SR) <> 0;
      finally
        FindClose(SR);
      end;
    end;
  end;
var
  aDir: String;
begin
  aDir := pDirName;
  if aDir = '' then
    Exit;
  if aDir[Length(aDir)] <> '\' then
    aDir := aDir + '\';
  ScanDir(aDir);
end;

function TSoundManager.GetRandomSound: string;
begin
  if Sounds.Count > 0 then
    result := Sounds[random(Sounds.Count)];
end;

function TSoundManager.PlaySound(pSoundName: String; pLoop, pWait: boolean): boolean;
var
  Info: BASS_CHANNELINFO;
  aChannel: HSTREAM;
begin
  if GetSoundIndex(pSoundName) > -1 then
  begin
    aChannel := HStream(Sounds.Objects[Sounds.IndexOf(pSoundName)]);

    // ChannelInfo lesen
    BASS_ChannelGetInfo(aChannel, Info);

    // Loop setzen / nicht setzen
    if ((not (Info.Flags = BASS_SAMPLE_LOOP)) AND (pLoop)) OR
        (Info.Flags = BASS_SAMPLE_LOOP) AND (not pLoop) then
      BASS_ChannelSetFlags(aChannel, BASS_SAMPLE_LOOP);

    // Abspielen
    BASS_ChannelPlay(aChannel, not pWait);

    result := true;
  end else
    result := false;
end;

function TSoundManager.PauseSound(pSoundName: string): boolean;
begin
  // Pause
  if GetSoundIndex(pSoundName) > -1 then
  begin
    BASS_ChannelPause(HStream(Sounds.Objects[Sounds.IndexOf(pSoundName)]));
    result := true;
  end else
    result := false;
end;

function TSoundManager.StopSound(pSoundName: string): boolean;
begin
  // Stop
  if GetSoundIndex(pSoundName) > -1 then
  begin
    BASS_ChannelStop(HStream(Sounds.Objects[Sounds.IndexOf(pSoundName)]));
    result := true;
  end else
    result := false;
end;

function TSoundManager.PlayRandomFromList(pSoundNames: TStrings; pLoop, pWait: boolean): string;
var
  tempname: string;
begin
  if pSoundNames.Count > 0 then
  begin
    tempname := pSoundNames[random(pSoundNames.Count)];
    PlaySound(tempname, ploop, pwait);
    result := tempname;
  end;
end;

Initialization
  randomize;

end.
