// Unit TextureManager
(**************************************************

- Enthlt: TTextureManager

**************************************************)

unit F2D_TextureManager;

interface

uses
  // Bentigte Units
  Windows,SysUtils, Graphics, IniFiles, dglOpenGL, glBitmap,
  F2D_BugTracker;

type
  // TextureManager legt Textureaddressierungen in Tgluint ab
  // Zugriff luft ber die HashedStringlist
  TTextureManager = class
    Textures: THashedStringList;
    FCurrentTex: string;
    // Bugtracker (siehe: UBugTracker)
    BugTracker: TBugTracker;
    constructor Create;
    destructor Destroy; override;    
    procedure Free;
  public
    // Textur hinzufgen
    procedure AddTexture(pFileName, pTextureName: String);
    procedure AddTextureFromBitmap(Bitmap: TBitmap; pTextureName: String);
    procedure AddTextureFromIcon(Icon: TIcon; pTextureName: String);
    // Alle Texturen eines Verzeichnisses hinzufgen (Filter: *.*)
    procedure AddTexturesInDir(pDirName, pFileMask: String; pSubDirs: Boolean);
    // Texture auswhlen (binden)
    procedure BindTexture(pTextureName: String);
    procedure UnBindTexture;
  end;

implementation

function IsPowerOfTwo(Value: Cardinal): Boolean;
begin  
  Result := (Value > 0) AND (Value AND (Value -1) = 0);  
end; 

constructor TTextureManager.Create;
begin
  inherited Create;

  // HashedStringList erstellen
  Textures:= THashedStringList.Create;
end;

destructor TTextureManager.Destroy;
begin
  // HashedStringList freigeben
  Textures.Free;

  inherited Destroy;
end;

procedure TTextureManager.Free;
begin
  if Self <> nil then
    Destroy;
end;

procedure TTextureManager.AddTexture(pFileName, pTextureName: String);
var t:TglBitmap2D;
begin
  // Prfen, ob die Textur nicht schon vorhanden ist
  if (Textures.IndexOf(pTextureName) = -1) then
    if (FileExists(pFilename)) then
    begin
      // Textur laden
      t := TglBitmap2D.Create(pFileName);

      t.DeleteTextureOnFree := False;
      t.FreeDataAfterGenTexture := False;
      if not (IsPowerOfTwo(t.Width) and IsPowerOfTwo(t.Height)) then
        t.Target:= GL_TEXTURE_RECTANGLE_EXT;
      t.GenTexture(True);
      Textures.AddObject(pTextureName, t);

      // BugTracker zum Log Datei schreiben veranlassen (O bedeutet erfolgreich)
      BugTracker.Write('O Texture: ' + pTextureName);
      BugTracker.Write('- Datei: ' + pFileName);
      BugTracker.Write('');
    end
    else
    begin
      // BugTracker zum Log Datei schreiben veranlassen (X bedeutet fehlgeschlagen)
      BugTracker.Write('X Texture: ' + pTextureName);
      BugTracker.Write('- Datei: ' + pFileName);
      BugTracker.Write('');
    end;
end;

procedure TTextureManager.AddTextureFromBitmap(Bitmap:TBitmap; pTextureName: String);
var t:TglBitmap2D;
begin
  // Prfen, ob die Textur nicht schon vorhanden ist
  if (Textures.IndexOf(pTextureName) = -1) then
  begin
    // Textur laden
    t := TglBitmap2D.Create;
    t.AssignFromBitmap(Bitmap);

    t.DeleteTextureOnFree := False;
    t.FreeDataAfterGenTexture := False;
    if not (IsPowerOfTwo(t.Width) and IsPowerOfTwo(t.Height)) then
      t.Target:= GL_TEXTURE_RECTANGLE_EXT;
    t.GenTexture(True);
    Textures.AddObject(pTextureName, t);

    // BugTracker zum Log Datei schreiben veranlassen (O bedeutet erfolgreich)
    BugTracker.Write('O Texture: ' + pTextureName);
    BugTracker.Write('- (TBitmap)');
    BugTracker.Write('');
  end;
end;

procedure TTextureManager.AddTextureFromIcon(Icon:TIcon; pTextureName: String);
type TRGBAArray = array[0..0] of TRGBQuad;
var t:TglBitmap2D;
  IconInfo: TIconInfo;
  Color,
  Mask: TBitmap;
  x,y:integer;
  clr,
  tex,
  alp:^TRGBAArray;
begin
  // Prfen, ob die Textur nicht schon vorhanden ist
  if (Textures.IndexOf(pTextureName) = -1) then
  begin
    // Textur laden
    t := TglBitmap2D.Create(glBitmapPosition(32,32),ifBGRA8);

    GetIconInfo(Icon.Handle, IconInfo);
    Color:= TBitmap.Create;
    Mask:= TBitmap.Create;
    try
      Color.Handle := IconInfo.hbmColor;
      Color.PixelFormat := pf32bit;

      mask.Handle := IconInfo.hbmMask;
      mask.PixelFormat := pf32bit;

      // bernahme des Masked Bereichs, ist entweder 0 (voll sichtbar) oder aber 255
      // (gar nicht sichtbar), deswegen auch das NOT davor weil der Alpha Kanal von Bitmaps
      // genau andersrum arbeitet.
      for y := 0 to Color.Height-1 do begin
        clr:= Color.ScanLine[y];
        tex:= t.Scanline[y];
        alp:= Mask.ScanLine[y];
        for x := 0 to Color.Width-1 do begin
          clr^[x].rgbReserved := NOT (alp^[x].rgbRed);
          tex[x]:= clr[x];
        end;
      end;
    finally
      Color.Free;
      Mask.Free;
    end;     

    t.DeleteTextureOnFree := False;
    t.FreeDataAfterGenTexture := False;
    if not (IsPowerOfTwo(t.Width) and IsPowerOfTwo(t.Height)) then
      t.Target:= GL_TEXTURE_RECTANGLE_EXT;
    t.GenTexture(True);
    Textures.AddObject(pTextureName, t);

    // BugTracker zum Log Datei schreiben veranlassen (O bedeutet erfolgreich)
    BugTracker.Write('O Texture: ' + pTextureName);
    BugTracker.Write('- (TIcon)');
    BugTracker.Write('');
  end;
end;



//Rekursive Dateisuche - Der Name des Sounds entspricht dann dem Dateinamen
procedure TTextureManager.AddTexturesInDir(pDirName, pFileMask: String; psubdirs: boolean);
  procedure ScanDir(const Directory: String);
    function CutExt(aStr: String): String;
    var
      I: Integer;
    begin
      Result := aStr;
      for I := Length(aStr) downto 1 do
        if aStr[i] = '.' then
        begin
          Result := Copy(aStr, 1, I - 1);
          Exit;
        end;
    end;
  var
    SR: TSearchRec;
  begin
    if FindFirst(Directory + pFileMask, faAnyFile - faDirectory, SR) = 0 then
    try
      repeat
        AddTexture(Directory + SR.Name, CutExt(SR.Name));
      until FindNext(SR) <> 0;
    finally
      FindClose(SR);
    end;
    if pSubDirs then
    begin
      if FindFirst(Directory + '*.*', faAnyFile, SR) = 0 then
      try
        repeat
          if ((SR.Attr AND faDirectory) = faDirectory) AND
              (SR.Name <> '.') AND (SR.Name <> '..') then
            ScanDir(Directory + SR.Name + '\');
        until FindNext(SR) <> 0;
      finally
        FindClose(SR);
      end;
    end;
  end;
var
  aDir: string;
begin
  aDir := pDirName;
  if aDir = '' then
    Exit;
  if aDir[Length(aDir)] <> '\' then
    aDir := aDir + '\';
  ScanDir(aDir);
end;

procedure TTextureManager.BindTexture(pTextureName: String);
var om:Integer;
begin
  if Textures.IndexOf(pTextureName)>=0 then
    // Texture Binden
    with TglBitmap2D(Textures.Objects[Textures.IndexOf(pTextureName)]) do begin
      Bind;
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 
      glGetIntegerv(GL_MATRIX_MODE,@om);
      glMatrixMode(GL_TEXTURE);
        glLoadIdentity;
        if Target=GL_TEXTURE_RECTANGLE_EXT then
          glScalef(Width,Height,1);
      glMatrixMode(om);
      FCurrentTex:= pTextureName;
    end
  else
    UnBindTexture;
end;

procedure TTextureManager.UnBindTexture;
begin
  if FCurrentTex>'' then
    TglBitmap2D(Textures.Objects[Textures.IndexOf(FCurrentTex)]).UnBind
  else
    glBindTexture(GL_TEXTURE_2D,0);
  FCurrentTex:= '';
end;

end.
