// Unit Core
(**************************************************

- Enthlt: TEngine

**************************************************)

unit F2D_Core;

interface

uses
  // Bentigte Units
  Windows, SysUtils, IniFiles, Forms, dglOpengl, glBitmap, graphics,
  F2D_Const,
  F2D_Vectors,
  F2D_Colors,
  F2D_TextureManager,
  F2D_Text,
  F2D_SoundFX,
  F2D_StateMachine,
  F2D_BugTracker;

type

  // TEngine
  // TextureManager - zum Laden / Verwalten von Texturen
  // SoundManager - zum Laden / Verwalten von Sounds und Musik
  // BugTracker - loggt (nicht) geladene Texturen und Sounds

  TEngine = class
    TextureManager:TTextureManager;
    SoundManager:  TSoundManager;
    StateMachine:  TStateMachine;
    BugTracker:    TBugTracker;
    FontManager:   TFontManager;  {--SH 20061111}


    constructor Create(pDebugMode: boolean);
    destructor Destroy; override;
    procedure Free;
  private
    RC:          HGLRC;           // Handle zum Zeichnen
    DC:          HDC;
    FFPS:        Single;          // FPS
    Initialised: Boolean;         // Initialisierungsvariable
    FStartTick:  Cardinal;        // Fr FPS berechnungen
    FFrames:     Integer;
    FTimeStep:   Single;          // Timestep
    FCalcTime:   Int64;           // Timestep-Berechungen mit QueryPerformanceCounter
    FFrequency:  Int64;
    function GetError: TGLUInt;
    function GetErrStr: string;
  public
    Width, Height: Integer;       // Breite des sichtbaren Clientbereichs
    X, Y:          Double;        // Position der Engine (ermglicht verschieben der "Kamera")
    zoom:          single;        // Zoomfaktor, default: 1
    Form:          TForm;         // Das Formular auf dem gezeichnet wird
    procedure DoBegin;            // DoBegin muss an den Anfang der Schleife / des Timers
    procedure DoEnd;              // DoEnd muss an das Ende der Schleife / des Timers
    procedure DoFullscreen;       // Lsst die Anwendung im Vollbildmodus laufen
    procedure DoScreenshot;       // Speichert das aktuelle Bild als bmp
    procedure Init(pForm: TForm); // Inptext: string);
    procedure TextOut(px, py: single; ptext: string;
               HCenter:boolean=false; VCenter:boolean=false); {--SH 20061111}
    procedure TextOutMultiline(px, py: single; ptext: string;
               HCenter:boolean=false; VCenter:boolean=false); {--SH 20061112}
    property TimeStep: Single read FTimeStep;     // Liefert den Zeitschritt zurck (Timebased Movement)
    property FPS: Single read FFPS;               // Liefert die Frames Per Second
    property Error : TGLUInt read GetError;
    property ErrorStr : string read GetErrStr;  
  end;

implementation

constructor TEngine.Create(pDebugMode: boolean);
begin
  inherited Create;

  // Klassen erstellen
  TextureManager := TTextureManager.Create;
  SoundManager   := TSoundManager.Create;
  StateMachine := TStateMachine.Create;
  FontManager  := TFontManager.Create(0);
  BugTracker     := TBugTracker.Create(pDebugMode);

  // BugTracker zuweisen
  TextureManager.BugTracker := BugTracker;
  SoundManager.BugTracker   := BugTracker;
  FontManager.BugTracker   := BugTracker;

  // Noch nicht initialisiert und Offset = 0
  Initialised := False;
  X := 0;
  Y := 0;
  zoom := 1;
end;

destructor TEngine.Destroy;
begin
  // Falls die Engine initialisiert wurde, mssen
  // die Display-Settings zurckgesetzt und der
  // RenderContext gelscht werden.

  if Initialised then
  begin
    DeactivateRenderingContext;
    wglDeleteContext(RC);
    ReleaseDC(Form.Handle, DC);
    ChangeDisplaySettings(DevMode(nil^), 0);
  end;

  // Klassen freigeben
  FontManager.Free;
  TextureManager.Free;
  SoundManager.Free;
  StateMachine.Free;
  BugTracker.Free;

  inherited Destroy;
end;

procedure TEngine.Free;
begin
  if Self <> nil then
    Destroy;
end;

procedure TEngine.Init(pForm: TForm);
begin
  // Formular zuweisen
  Form := pForm;

  // Die Breite zum Startzeitpunkt ermitteln (fr resizen bentigt)
  Width  := Form.ClientWidth;
  Height := Form.ClientHeight;

  // OpenGL Initialisieren
  InitOpenGL;

  // Handle holen
  DC := GetDC(Form.Handle);

  // RenderContext erstellen und aktivieren
  RC := CreateRenderingContext(DC, [opDoubleBuffered], 32, 24, 0, 0, 0, 0);
  ActivateRenderingContext(DC, RC);

  // Aktivieren von Texturen, Tiefen-Tests, etc.
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LESS);
  glEnable(GL_CULL_FACE);
  glCullFace(GL_FRONT);

  // Lschfarbe auf Schwarz setzen
  glClearColor(0, 0, 0, 0);

  // Zeitmessungen fr FPS und TimeStep beginnen
  FStartTick := GetTickCount;
  QueryPerformanceFrequency(FFrequency);
  QueryPerformanceCounter(FCalcTime);
  
{--SH 20061111 BEGIN}
  FontManager.FDC:= DC;
  FontManager.AddFont('default',Form.Font);
  FontManager.BindFont('default',clwhite);
{--SH 20061111 END}

  // Initialisierung abgeschlossen
  Initialised := True;
end;

procedure TEngine.DoBegin;
var
  CurrTime: Int64;
begin
  // Berechnungen fr TimeStep und FPS
  QueryPerformanceCounter(Currtime);
  FTimeStep := (CurrTime - FCalcTime) / FFrequency;
  FCalcTime := CurrTime;

  // Bild leeren
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  // Projektion ausrichten
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  glViewport(0, 0, Form.ClientWidth, Form.ClientHeight);

  // Setzt den Ursprung links oben an
  glOrtho(0, Form.ClientWidth, Form.ClientHeight, 0, 0, maxdepth);

  // ModelMatrix ausrichten
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;

  glTranslatef(form.width / 2, form.Height / 2, 0);
  //Zoom
  glScalef(zoom, zoom, 1);
  glTranslatef(-form.width / 2, -form.Height / 2, 0);

  // Setz den Zeichenpunkt auf das Offset
  glTranslatef(x, y, 0);
end;

procedure TEngine.DoEnd;
begin
  // Aktiviert GL_ALWAYS (immer blenden)
  glDepthFunc(GL_ALWAYS);

  //SwapBuffers
  SwapBuffers(DC);

  //FrameCount erhhen - FPS berechnen
  inc(FFrames);

  if GetTickCount - FStartTick > 499 then
  begin
    FFPS       := FFrames / (GetTickCount - FStartTick) * 1000;
    FFrames    := 0;
    FStartTick := GetTickCount;
  end;
end;

procedure TEngine.DoFullscreen;
begin
  with Form do
  begin
    //Ecke oben links
    top := 0;
    left := 0;
    //Volle Gre
    clientwidth := screen.width;
    clientheight := screen.height;
    //Status maximiert
    Windowstate := wsMaximized;
    //Kein Rahmen
    SetWindowLong(Form.Handle, GWL_STYLE, WS_BORDER);
    //StayOnTop
    SetWindowPos(form.handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE or SWP_NOSIZE);
  end;
end;

procedure TEngine.DoScreenshot;
type
 BITMAPFILEHEADER = packed record
    bfType: Word;
    bfSize: DWORD;
    bfReserved1: Word;
    bfReserved2: Word;
    bfOffBits: DWORD;
  end;
 
 BITMAPINFOHEADER = packed record
    biSize: DWORD;
    biWidth: Longint;
    biHeight: Longint;
    biPlanes: Word;
    biBitCount: Word;
    biCompression: DWORD;
    biSizeImage: DWORD;
    biXPelsPerMeter: Longint;
    biYPelsPerMeter: Longint;
    biClrUsed: DWORD;
    biClrImportant: DWORD;
  end;
 var
   f: file;
   FileInfo: BITMAPINFOHEADER;
   FileHeader: BITMAPFILEHEADER;
   pPicData:Pointer;
   Viewport: array[0..3] of integer;
   path: string;
   i: integer;
begin
  ZeroMemory(@FileHeader, SizeOf(BITMAPFILEHEADER));
  ZeroMemory(@FileInfo, SizeOf(BITMAPINFOHEADER));

  glGetIntegerv(GL_VIEWPORT, @Viewport);

  //Header
  FileHeader.bfType := 19778; //$4D42 = 'BM'
  FileHeader.bfOffBits := SizeOf(BITMAPINFOHEADER)+SizeOf(BITMAPFILEHEADER);

  //FileInfo
  FileInfo.biSize := SizeOf(BITMAPINFOHEADER);
  FileInfo.biWidth := Viewport[2];
  FileInfo.biHeight := Viewport[3];
  FileInfo.biPlanes := 1;
  FileInfo.biBitCount := 32;
  FileInfo.biSizeImage := FileInfo.biWidth*FileInfo.biHeight*(FileInfo.biBitCount div 8);
  FileHeader.bfSize := FileHeader.bfOffBits + FileInfo.biSizeImage;

  GetMem(pPicData, FileInfo.biSizeImage);
  try
    glReadPixels(0, 0, Viewport[2], Viewport[3], GL_BGRA, GL_UNSIGNED_BYTE, pPicData);
    //Dateiname
    i := 0;
    path := extractfilepath(paramstr(0));
    while fileexists(path + 'screen_' + inttostr(i) + '.bmp') do
      inc(i);
    AssignFile(f, path + 'screen_' + inttostr(i) + '.bmp');
    Rewrite(f, 1);
    try
      //Schreiben
      BlockWrite(F, FileHeader, SizeOf(BITMAPFILEHEADER));
      BlockWrite(F, FileInfo, SizeOf(BITMAPINFOHEADER));
      BlockWrite(F, pPicData^, FileInfo.biSizeImage );
    finally
      //Schlieen
      CloseFile(f);
    end;
  finally
    //Freigeben
    FreeMem(pPicData, FileInfo.biSizeImage);
  end;
end;

procedure TEngine.TextOut(px, py: single; ptext: string; HCenter,
  VCenter: boolean);
var c:TSize;
begin
  c:= FontManager.TextSize(FontManager.SelectedFont,ptext);
  if HCenter then
    px:= px- c.cx/2;
  if not VCenter then
    py:= py- c.cy/2;
    
  glPushMatrix;
    glRasterPos3f(pX,pY,0);
    glCallLists(Length(pText), GL_UNSIGNED_BYTE, PChar(pText));//Entsprechende Listen aufrufen
  glPopMatrix;
end;

procedure TEngine.TextOutMultiline(px, py: single; ptext: string; HCenter,
  VCenter: boolean);
var t:string;
    c:single;
begin
  c:= FontManager.GeneralTextHeight(FontManager.SelectedFont);
  while ptext>'' do begin
    if pos(#13,ptext)>0 then begin
      t:= copy(ptext,1,pos(#13,ptext)-1);
      delete(ptext,1,length(t)+1);
    end else begin
      t:= pText;
      ptext:= '';
    end;
    TextOut(px,py,t,HCenter,VCenter);
    py:= py+c*LineSpacing;
  end;
end;

function TEngine.GetError: TGLUInt;
begin
  Result:= glGetError;
end;

function TEngine.GetErrStr: string;
begin
  Result:= gluErrorString(Error);
end;

end.
